@@ -10,7 +10,8 @@
 #ifndef CM_MEMMAP_H
 #define CM_MEMMAP_H 1
 
-#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | 'D'<<24)
+#define CM_CONFIG_DATA_VERSION  3
+#define CM_CONFIG_DATA_MAGIC            ('A' | 'F'<<8 | 'S'<<16 | CM_CONFIG_DATA_VERSION<<24)
 
 typedef struct cm_config_data {
     afs_uint32          size;
@@ -46,19 +47,32 @@ typedef struct cm_config_data {
     cm_fid_t            rootFid;
     cm_scache_t *       rootSCachep;
     cm_scache_t         fakeSCache;
-    afs_uint32          fakeDirVersion;
+    afs_uint64          fakeDirVersion;
 
     cm_aclent_t *       aclLRUp;
     cm_aclent_t	*       aclLRUEndp;
 
-    cm_scache_t	**      hashTablep;
-    afs_uint32		hashTableSize;
+    cm_scache_t	**      scacheHashTablep;
+    afs_uint32		scacheHashTableSize;
 
+    cm_scache_t *       allSCachesp;
     afs_uint32		currentSCaches;
     afs_uint32          maxSCaches;
     cm_scache_t *       scacheLRUFirstp;
     cm_scache_t *       scacheLRULastp;
 
+    cm_cell_t   **      cellNameHashTablep;
+    cm_cell_t   **      cellIDHashTablep;
+    afs_uint32          cellHashTableSize;
+
+    cm_volume_t **      volumeNameHashTablep;
+    cm_volume_t **      volumeRWIDHashTablep;
+    cm_volume_t **      volumeROIDHashTablep;
+    cm_volume_t **      volumeBKIDHashTablep;
+    afs_uint32          volumeHashTableSize;
+    cm_volume_t *       volumeLRUFirstp;
+    cm_volume_t *       volumeLRULastp;
+
     cm_nc_t 	*       ncfreelist;
     cm_nc_t 	*       nameCache;
     cm_nc_t 	**      nameHash; 
@@ -67,7 +81,7 @@ typedef struct cm_config_data {
     cm_buf_t    *       buf_freeListEndp;
     cm_buf_t	*       buf_dirtyListp;
     cm_buf_t    *       buf_dirtyListEndp;
-    cm_buf_t	**      buf_hashTablepp;
+    cm_buf_t	**      buf_scacheHashTablepp;
     cm_buf_t	**      buf_fileHashTablepp;
     cm_buf_t	*       buf_allp;
     afs_uint64		buf_nbuffers;
@@ -102,5 +116,5 @@ PSECURITY_ATTRIBUTES CreateCacheFileSA();
 VOID  FreeCacheFileSA(PSECURITY_ATTRIBUTES psa);
 int   cm_ShutdownMappedMemory(void);
 int   cm_ValidateMappedMemory(char * cachePath);
-int   cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD chunkSize, afs_uint64 cacheBlocks );
+int   cm_InitMappedMemory(DWORD virtualCache, char * cachePath, DWORD stats, DWORD maxVols, DWORD maxCells, DWORD chunkSize, afs_uint64 cacheBlocks, afs_uint32 blockSize);
 #endif /* CM_MEMMAP_H */
\ No newline at end of file
