@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vnodeops.c,v 1.20.2.3.2.1 2005/10/12 06:06:47 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/SOLARIS/osi_vnodeops.c,v 1.28.2.4 2008/03/17 15:28:33 shadow Exp $");
 
 /*
  * SOLARIS/osi_vnodeops.c
@@ -47,6 +47,9 @@ RCSID
 #include <vm/seg_map.h>
 #include <vm/seg_vn.h>
 #include <vm/rm.h>
+#if defined(AFS_SUN511_ENV)
+#include <sys/vfs_opreg.h>
+#endif
 #include <sys/modctl.h>
 #include <sys/syscall.h>
 #include <sys/debug.h>
@@ -266,7 +269,7 @@ afs_GetOnePage(vp, off, alen, protp, pl, plsz, seg, addr, rw, acred)
     register struct dcache *tdc;
     int i, s, pexists;
     int slot;
-    afs_size_t offset = 0, nlen;
+    afs_size_t offset, nlen = 0;
     struct vrequest treq;
     afs_int32 mapForRead = 0, Code = 0;
     u_offset_t toffset;
@@ -457,7 +460,11 @@ afs_GetOnePage(vp, off, alen, protp, pl, plsz, seg, addr, rw, acred)
 	    buf = pageio_setup(page, PAGESIZE, vp, B_READ);	/* allocate a buf structure */
 	    buf->b_edev = 0;
 	    buf->b_dev = 0;
+#if defined(AFS_SUN56_ENV)
+	    buf->b_lblkno = lbtodb(toffset);
+#else
 	    buf->b_blkno = btodb(toffset);
+#endif
 	    bp_mapin(buf);	/* map it in to our address space */
 
 	    AFS_GLOCK();
@@ -685,7 +692,11 @@ afs_putapage(struct vnode *vp, struct page *pages,
 	    return (ENOMEM);
 
 	tbuf->b_dev = 0;
+#if defined(AFS_SUN56_ENV)
+	tbuf->b_lblkno = lbtodb(pages->p_offset);
+#else
 	tbuf->b_blkno = btodb(pages->p_offset);
+#endif
 	bp_mapin(tbuf);
 	AFS_GLOCK();
 	afs_Trace4(afs_iclSetp, CM_TRACE_PAGEOUTONE, ICL_TYPE_LONG, avc,
@@ -743,7 +754,7 @@ afs_nfsrdwr(avc, auio, arw, ioflag, acred)
 
     afs_Trace4(afs_iclSetp, CM_TRACE_VMRW, ICL_TYPE_POINTER, (afs_int32) avc,
 	       ICL_TYPE_LONG, (arw == UIO_WRITE ? 1 : 0), ICL_TYPE_OFFSET,
-	       ICL_HANDLE_OFFSET(auio->uio_offset), ICL_TYPE_OFFSET,
+	       ICL_HANDLE_OFFSET(auio->uio_loffset), ICL_TYPE_OFFSET,
 	       ICL_HANDLE_OFFSET(auio->uio_resid));
 
 #ifndef AFS_64BIT_CLIENT
@@ -762,9 +773,9 @@ afs_nfsrdwr(avc, auio, arw, ioflag, acred)
      */
     afs_MaybeWakeupTruncateDaemon();
     while ((arw == UIO_WRITE)
-	   && (afs_blocksUsed > (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100)) {
+	   && (afs_blocksUsed > PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks))) {
 	if (afs_blocksUsed - afs_blocksDiscarded >
-	    (CM_WAITFORDRAINPCT * afs_cacheBlocks) / 100) {
+	    PERCENT(CM_WAITFORDRAINPCT, afs_cacheBlocks)) {
 	    afs_WaitForCacheDrain = 1;
 	    afs_osi_Sleep(&afs_WaitForCacheDrain);
 	}
@@ -782,12 +793,13 @@ afs_nfsrdwr(avc, auio, arw, ioflag, acred)
 
     /* adjust parameters when appending files */
     if ((ioflag & IO_APPEND) && arw == UIO_WRITE) {
-#if	defined(AFS_SUN56_ENV)
-	auio->uio_loffset = 0;
-#endif
+#if defined(AFS_SUN56_ENV)
+	auio->uio_loffset = avc->m.Length;	/* write at EOF position */
+#else
 	auio->uio_offset = avc->m.Length;	/* write at EOF position */
+#endif
     }
-    if (auio->uio_offset < 0 || (auio->uio_offset + auio->uio_resid) < 0) {
+    if (auio->afsio_offset < 0 || (auio->afsio_offset + auio->uio_resid) < 0) {
 	ReleaseWriteLock(&avc->lock);
 	afs_BozonUnlock(&avc->pvnLock, avc);
 	return EINVAL;
@@ -886,8 +898,9 @@ afs_nfsrdwr(avc, auio, arw, ioflag, acred)
 	 * call it with an offset based on blocks smaller than MAXBSIZE
 	 * (implying that it should be named BSIZE, since it is clearly both a
 	 * max and a min). */
-	size = auio->afsio_resid;	/* transfer size */
-	fileBase = auio->afsio_offset;	/* start file position for xfr */
+	size = auio->afsio_resid;       /* transfer size */     
+	fileBase = ((arw == UIO_READ) && (origLength < auio->uio_offset)) ? 
+	    origLength : auio->afsio_offset;  /* start file position for xfr */
 	pageBase = fileBase & ~(MAXBSIZE - 1);	/* file position of the page */
 	pageOffset = fileBase & (MAXBSIZE - 1);	/* xfr start's offset within page */
 	tsize = MAXBSIZE - pageOffset;	/* how much more fits in this page */
@@ -1233,7 +1246,13 @@ afs_seek(vnp, ooff, noffp)
 {
     register int code = 0;
 
-    if ((*noffp < 0 || *noffp > MAXOFF_T))
+#ifndef AFS_64BIT_CLIENT
+# define __MAXOFF_T MAXOFF_T
+#else
+# define __MAXOFF_T MAXOFFSET_T
+#endif
+
+    if ((*noffp < 0 || *noffp > __MAXOFF_T))
 	code = EINVAL;
     return code;
 }
@@ -1385,6 +1404,25 @@ afs_dumpctl(vp, i
 }
 
 #ifdef	AFS_SUN54_ENV
+#ifdef	AFS_SUN511_ENV
+extern void
+afs_dispose(struct vnode *vp, struct page *p, int fl, int dn, struct cred *cr, struct caller_context_t *ct)
+{
+    fs_dispose(vp, p, fl, dn, cr,ct);
+}
+
+int
+afs_setsecattr(struct vnode *vp, vsecattr_t *vsecattr, int flag, struct cred *creds, struct caller_context_t *ct)
+{
+    return ENOSYS;
+}
+
+int
+afs_getsecattr(struct vnode *vp, vsecattr_t *vsecattr, int flag, struct cred *creds, struct caller_context_t *ct)
+{
+  return fs_fab_acl(vp, vsecattr, flag, creds,ct);
+}
+#else
 extern void
 afs_dispose(vp, p, fl, dn, cr)
      struct vnode *vp;
@@ -1415,6 +1453,7 @@ afs_getsecattr(vp, vsecattr, flag, creds)
     return fs_fab_acl(vp, vsecattr, flag, creds);
 }
 #endif
+#endif
 
 #ifdef	AFS_GLOBAL_SUNLOCK
 extern int gafs_open(), gafs_close(), afs_ioctl(), gafs_access();
@@ -1425,52 +1464,102 @@ extern int gafs_fid(), gafs_readlink(), fs_setfl(), afs_pathconf();
 extern int afs_lockctl();
 extern void gafs_inactive();
 
-#ifdef AFS_SUN510_ENV
-struct fs_operation_def afs_vnodeops_template[] = {
-    { VOPNAME_OPEN,		gafs_open },
-    { VOPNAME_CLOSE,		gafs_close },
-    { VOPNAME_READ,		afs_vmread },
-    { VOPNAME_WRITE,		afs_vmwrite },
-    { VOPNAME_IOCTL,		afs_ioctl },
-    { VOPNAME_SETFL,		fs_setfl },
-    { VOPNAME_GETATTR,		gafs_getattr },
-    { VOPNAME_SETATTR,		gafs_setattr },
-    { VOPNAME_ACCESS,		gafs_access },
-    { VOPNAME_LOOKUP,		gafs_lookup },
-    { VOPNAME_CREATE,		gafs_create },
-    { VOPNAME_REMOVE,		gafs_remove },
-    { VOPNAME_LINK,		gafs_link },
-    { VOPNAME_RENAME,		gafs_rename },
-    { VOPNAME_MKDIR,		gafs_mkdir },
-    { VOPNAME_RMDIR,		gafs_rmdir },
-    { VOPNAME_READDIR,		gafs_readdir },
-    { VOPNAME_SYMLINK,		gafs_symlink },   
-    { VOPNAME_READLINK,		gafs_readlink },
-    { VOPNAME_FSYNC,		gafs_fsync },
-    { VOPNAME_INACTIVE,		gafs_inactive },
-    { VOPNAME_FID,		gafs_fid },
-    { VOPNAME_RWLOCK,		afs_rwlock },
-    { VOPNAME_RWUNLOCK,		afs_rwunlock },
-    { VOPNAME_SEEK,		afs_seek },
-    { VOPNAME_CMP,		afs_cmp },
-    { VOPNAME_FRLOCK,		afs_frlock },
-    { VOPNAME_SPACE,		afs_space },
-    { VOPNAME_REALVP,		afs_realvp },
-    { VOPNAME_GETPAGE,		afs_getpage },
-    { VOPNAME_PUTPAGE,		afs_putpage },
-    { VOPNAME_MAP,		afs_map },
-    { VOPNAME_ADDMAP,		afs_addmap },
-    { VOPNAME_DELMAP,		afs_delmap },
-    { VOPNAME_POLL,		fs_poll },
-    { VOPNAME_DUMP,		afs_dump },
-    { VOPNAME_PATHCONF,		afs_pathconf },
-    { VOPNAME_PAGEIO,		afs_pageio },
-    { VOPNAME_DUMPCTL,		afs_dumpctl },   
-    { VOPNAME_DISPOSE,		afs_dispose },
-    { VOPNAME_GETSECATTR,	afs_getsecattr },
-    { VOPNAME_SETSECATTR,	afs_setsecattr },
-    { VOPNAME_SHRLOCK,		fs_shrlock },
-    NULL,
+#if defined(AFS_SUN511_ENV)
+/* The following list must always be NULL-terminated */
+const fs_operation_def_t afs_vnodeops_template[] = {
+    VOPNAME_OPEN,		{ .vop_open = gafs_open },
+    VOPNAME_CLOSE,		{ .vop_close = gafs_close },
+    VOPNAME_READ,		{ .vop_read = afs_vmread },
+    VOPNAME_WRITE,		{ .vop_write = afs_vmwrite },
+    VOPNAME_IOCTL,		{ .vop_ioctl = afs_ioctl },
+    VOPNAME_SETFL,		{ .vop_setfl = fs_setfl },
+    VOPNAME_GETATTR,		{ .vop_getattr = gafs_getattr },
+    VOPNAME_SETATTR,		{ .vop_setattr = gafs_setattr },
+    VOPNAME_ACCESS,		{ .vop_access = gafs_access },
+    VOPNAME_LOOKUP,		{ .vop_lookup = gafs_lookup },
+    VOPNAME_CREATE,		{ .vop_create = gafs_create },
+    VOPNAME_REMOVE,		{ .vop_remove = gafs_remove },
+    VOPNAME_LINK,		{ .vop_link = gafs_link },
+    VOPNAME_RENAME,		{ .vop_rename = gafs_rename },
+    VOPNAME_MKDIR,		{ .vop_mkdir = gafs_mkdir },
+    VOPNAME_RMDIR,		{ .vop_rmdir = gafs_rmdir },
+    VOPNAME_READDIR,		{ .vop_readdir = gafs_readdir },
+    VOPNAME_SYMLINK,		{ .vop_symlink = gafs_symlink },   
+    VOPNAME_READLINK,		{ .vop_readlink = gafs_readlink },
+    VOPNAME_FSYNC,		{ .vop_fsync = gafs_fsync },
+    VOPNAME_INACTIVE,		{ .vop_inactive = gafs_inactive },
+    VOPNAME_FID,		{ .vop_fid = gafs_fid },
+    VOPNAME_RWLOCK,		{ .vop_rwlock = afs_rwlock },
+    VOPNAME_RWUNLOCK,		{ .vop_rwunlock = afs_rwunlock },
+    VOPNAME_SEEK,		{ .vop_seek = afs_seek },
+    VOPNAME_CMP,		{ .vop_cmp = afs_cmp },
+    VOPNAME_FRLOCK,		{ .vop_frlock = afs_frlock },
+    VOPNAME_SPACE,		{ .vop_space = afs_space },
+    VOPNAME_REALVP,		{ .vop_realvp = afs_realvp },
+    VOPNAME_GETPAGE,		{ .vop_getpage = afs_getpage },
+    VOPNAME_PUTPAGE,		{ .vop_putpage = afs_putpage },
+    VOPNAME_MAP,		{ .vop_map = afs_map },
+    VOPNAME_ADDMAP,		{ .vop_addmap = afs_addmap },
+    VOPNAME_DELMAP,		{ .vop_delmap = afs_delmap },
+    VOPNAME_POLL,		{ .vop_poll = fs_poll },
+    VOPNAME_PATHCONF,		{ .vop_pathconf = afs_pathconf },
+    VOPNAME_PAGEIO,		{ .vop_pageio = afs_pageio },
+    VOPNAME_DUMP,		{ .vop_dump = afs_dump },
+    VOPNAME_DUMPCTL,		{ .vop_dumpctl = afs_dumpctl },   
+    VOPNAME_DISPOSE,		{ .vop_dispose = afs_dispose },
+    VOPNAME_GETSECATTR,		{ .vop_getsecattr = afs_getsecattr },
+    VOPNAME_SETSECATTR, 	{ .vop_setsecattr = afs_setsecattr },
+    VOPNAME_SHRLOCK,		{ .vop_shrlock = fs_shrlock },
+    NULL,			NULL
+};
+vnodeops_t *afs_ops;
+#elif defined(AFS_SUN510_ENV)
+/* The following list must always be NULL-terminated */
+const fs_operation_def_t afs_vnodeops_template[] = {
+    VOPNAME_OPEN,		gafs_open,
+    VOPNAME_CLOSE,		gafs_close,
+    VOPNAME_READ,		afs_vmread,
+    VOPNAME_WRITE,		afs_vmwrite,
+    VOPNAME_IOCTL,		afs_ioctl,
+    VOPNAME_SETFL,		fs_setfl,
+    VOPNAME_GETATTR,		gafs_getattr,
+    VOPNAME_SETATTR,		gafs_setattr,
+    VOPNAME_ACCESS,		gafs_access,
+    VOPNAME_LOOKUP,		gafs_lookup,
+    VOPNAME_CREATE,		gafs_create,
+    VOPNAME_REMOVE,		gafs_remove,
+    VOPNAME_LINK,		gafs_link,
+    VOPNAME_RENAME,		gafs_rename,
+    VOPNAME_MKDIR,		gafs_mkdir,
+    VOPNAME_RMDIR,		gafs_rmdir,
+    VOPNAME_READDIR,		gafs_readdir,
+    VOPNAME_SYMLINK,		gafs_symlink,   
+    VOPNAME_READLINK,		gafs_readlink,
+    VOPNAME_FSYNC,		gafs_fsync,
+    VOPNAME_INACTIVE,		gafs_inactive,
+    VOPNAME_FID,		gafs_fid,
+    VOPNAME_RWLOCK,		afs_rwlock,
+    VOPNAME_RWUNLOCK,		afs_rwunlock,
+    VOPNAME_SEEK,		afs_seek,
+    VOPNAME_CMP,		afs_cmp,
+    VOPNAME_FRLOCK,		afs_frlock,
+    VOPNAME_SPACE,		afs_space,
+    VOPNAME_REALVP,		afs_realvp,
+    VOPNAME_GETPAGE,		afs_getpage,
+    VOPNAME_PUTPAGE,		afs_putpage,
+    VOPNAME_MAP,		afs_map,
+    VOPNAME_ADDMAP,		afs_addmap,
+    VOPNAME_DELMAP,		afs_delmap,
+    VOPNAME_POLL,		fs_poll,
+    VOPNAME_DUMP,		afs_dump,
+    VOPNAME_PATHCONF,		afs_pathconf,
+    VOPNAME_PAGEIO,		afs_pageio,
+    VOPNAME_DUMPCTL,		afs_dumpctl,   
+    VOPNAME_DISPOSE,		afs_dispose,
+    VOPNAME_GETSECATTR,       afs_getsecattr,
+    VOPNAME_SETSECATTR, 	afs_setsecattr,
+    VOPNAME_SHRLOCK,		fs_shrlock,
+    NULL,                     NULL
 };
 struct vnodeops *afs_ops;
 #else
@@ -1682,11 +1771,13 @@ gafs_rename(aodp, aname1, andp, aname2, acred)
 	    struct vnode *vp = AFSTOV(avcp), *pvp = AFSTOV(andp);
 	    
 	    mutex_enter(&vp->v_lock);
-	    kmem_free(vp->v_path, strlen(vp->v_path) + 1);
-	    vp->v_path = NULL;
+	    if (vp->v_path != NULL) {
+		kmem_free(vp->v_path, strlen(vp->v_path) + 1);
+		vp->v_path = NULL;
+	    }
 	    mutex_exit(&vp->v_lock);
-	    VN_SETPATH(afs_globalVp, pvp, vp, aname2, strlen(aname2));
-	    
+	    vn_setpath(afs_globalVp, pvp, vp, aname2, strlen(aname2));
+
 	    AFS_RELE(avcp);
 	}
     }
