@@ -86,11 +86,13 @@ extern int afs_shuttingdown;
 #define	AFS_NRXPACKETS	80
 #define	AFS_RXDEADTIME	50
 #define AFS_HARDDEADTIME	120
+#define	AFS_IDLEDEADTIME	50
 #define AFS_BLKBITS	12
 #define AFS_BLKSIZE	(1 << AFS_BLKBITS)
 
 extern afs_int32 afs_rx_deadtime;
 extern afs_int32 afs_rx_harddead;
+extern afs_int32 afs_rx_idledead;
 
 struct sysname_info {
     char *name;
@@ -186,6 +188,9 @@ struct vrequest {
     char volumeError;		/* encountered a missing or busy volume */
     char networkError;		/* encountered network problems */
     char permWriteError;	/* fileserver returns permenent error. */
+    char tokenError;            /* a token error other than expired. */
+    char idleError;             /* the server idled too long */
+    char skipserver[MAXHOSTS];
 };
 #define VOLMISSING 1
 #define VOLBUSY 2
@@ -270,6 +275,9 @@ enum { AFS_GCPAGS_NOTCOMPILED = 0, AFS_GCPAGS_OK =
 extern afs_int32 afs_gcpags;
 extern afs_int32 afs_gcpags_procsize;
 extern afs_int32 afs_bkvolpref;
+extern char afs_cachebasedir[1024];
+extern afs_int32 afs_numcachefiles;
+extern afs_int32 afs_numfilesperdir;
 
 struct unixuser {
     struct unixuser *next;	/* next hash pointer */
@@ -920,6 +928,9 @@ struct afs_fheader {
 #endif
 #endif
 
+
+#ifdef KERNEL
+/* it does not compile outside kernel */
 struct buffer {
   afs_int32 fid;              /* is adc->index, the cache file number */
   afs_inode_t inode;          /* is adc->f.inode, the inode number of the cac\
@@ -947,6 +958,7 @@ struct fcache {
     afs_int32 chunkBytes;	/* Num bytes in this chunk */
     char states;		/* Has this chunk been modified? */
 };
+#endif
 
 /* magic numbers to specify the cache type */
 
@@ -1225,5 +1237,4 @@ struct afs_fakestat_state {
 };
 
 extern int afs_fakestat_enable;
-
 #endif /* _AFS_H_ */
