@@ -79,7 +79,11 @@
 #define SYS_NAME       "alpha_linux_26"
 #define SYS_NAME_ID    SYS_NAME_ID_alpha_linux_26
 
-
+#ifdef __GLIBC__
+#if (__GLIBC__ > 2) || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 3)
+#define USE_UCONTEXT
+#endif
+#endif
 #endif /* AFS_PARAM_H */
 				     
 #else /* !defined(UKERNEL) */
