@@ -36,7 +36,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxgen/rpc_hout.c,v 1.7.10.2 2005/10/15 21:53:58 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/rxgen/rpc_hout.c,v 1.10.4.3 2008/02/05 17:18:42 jaltman Exp $");
 
 #include <stdio.h>
 #include <string.h>
@@ -192,7 +192,7 @@ define_printed(proc_list * stop, version_list * start)
 	}
     }
     abort();
-    /* NOTREACHED */
+    return 0;/* NOTREACHED */
 }
 
 
@@ -223,8 +223,10 @@ psproc1(definition * defp, int callTconnF, char *type, char *prefix,
     f_print(fout, "\nextern %s %s%s%s(\n", type, prefix, defp->pc.proc_prefix,
 	    defp->pc.proc_name);
 
-    if (callTconnF) {
+    if (callTconnF == 1) {
 	f_print(fout, "\t/*IN */ struct rx_call *z_call");
+    } else if (callTconnF == 2) {
+	f_print(fout, "\tregister struct ubik_client *aclient, afs_int32 aflags");
     } else {
 	f_print(fout, "\t/*IN */ struct rx_connection *z_conn");
     }
@@ -266,9 +268,12 @@ psprocdef(definition * defp)
 		(1 << DEF_INPARAM) | (1 << DEF_INOUTPARAM));
 	psproc1(defp, 1, "int", "End",
 		(1 << DEF_OUTPARAM) | (1 << DEF_INOUTPARAM));
-    } else {
-	psproc1(defp, 0, "int", "", 0xFFFFFFFF);
-    }
+    } 
+    if (!(!multi_flag && split_flag))
+        psproc1(defp, 0, "int", "", 0xFFFFFFFF);
+
+    if (uflag && !kflag)
+	psproc1(defp, 2, "int", "ubik_", 0xFFFFFFFF);
 
     if (*ServerPrefix)
 	psproc1(defp, 1, "afs_int32", ServerPrefix, 0xFFFFFFFF);
