@@ -11,10 +11,11 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.15 2005/06/18 23:23:46 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vlserver/vlprocs.c,v 1.15.4.3 2008/04/02 19:51:57 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <string.h>
 #include <lock.h>
 #include <afs/afsutil.h>
 #include <ubik.h>
@@ -25,13 +26,6 @@ RCSID
 #else
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <afs/keys.h>
 #include "vlserver.h"
 #include "afs/audit.h"
@@ -61,17 +55,17 @@ static char rxinfo_str[128];	/* Need rxinfo string to be non-local */
 static int put_attributeentry();
 static int put_nattributeentry();
 static int RemoveEntry();
-static ReleaseEntry();
+static void ReleaseEntry();
 static int check_vldbentry();
 static int check_nvldbentry();
 static int vldbentry_to_vlentry();
 static int nvldbentry_to_vlentry();
 static get_vldbupdateentry();
 static int repsite_exists();
-static repsite_compress();
-static vlentry_to_vldbentry();
-static vlentry_to_nvldbentry();
-static vlentry_to_uvldbentry();
+static void repsite_compress();
+static void vlentry_to_vldbentry();
+static void vlentry_to_nvldbentry();
+static void vlentry_to_uvldbentry();
 static int InvalidVolname();
 static int InvalidVoltype();
 static int InvalidOperation();
@@ -1141,7 +1135,9 @@ SVL_ListAttributes(rxcall, attributes, nentries, vldbentries)
 	while (nextblockindex =
 	       NextEntry(trans, nextblockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 	    match = 0;
@@ -1279,7 +1275,9 @@ SVL_ListAttributesN(rxcall, attributes, nentries, vldbentries)
 	while (nextblockindex =
 	       NextEntry(trans, nextblockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1461,7 +1459,9 @@ SVL_ListAttributesN2(rxcall, attributes, name, startindex, nentries,
 	blockindex = startindex;
 	while (blockindex = NextEntry(trans, blockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1683,7 +1683,9 @@ SVL_LinkedList(rxcall, attributes, nentries, vldbentries)
 	    match = 0;
 
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -1809,7 +1811,9 @@ SVL_LinkedListN(rxcall, attributes, nentries, vldbentries)
 	    match = 0;
 
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 
@@ -2562,7 +2566,7 @@ RemoveEntry(trans, entryptr, tentry)
     return 0;
 }
 
-static
+static void
 ReleaseEntry(tentry, releasetype)
      struct nvlentry *tentry;
      afs_int32 releasetype;
@@ -2867,7 +2871,7 @@ repsite_exists(VlEntry, server, partition)
 
 
 /* Repsite table compression: used when deleting a repsite entry so that all active repsite entries are on the top of the table. */
-static
+static void
 repsite_compress(VlEntry, offset)
      struct nvlentry *VlEntry;
      int offset;
@@ -2888,7 +2892,7 @@ repsite_compress(VlEntry, offset)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_vldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct vldbentry *VldbEntry;
@@ -2930,7 +2934,7 @@ vlentry_to_vldbentry(VlEntry, VldbEntry)
 
 
 /* Convert from the internal (compacted) vldb entry to the external representation used by the interface. */
-static
+static void
 vlentry_to_nvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct nvldbentry *VldbEntry;
@@ -2970,7 +2974,7 @@ vlentry_to_nvldbentry(VlEntry, VldbEntry)
     VldbEntry->flags = VlEntry->flags;
 }
 
-static
+static void
 vlentry_to_uvldbentry(VlEntry, VldbEntry)
      struct nvlentry *VlEntry;
      struct uvldbentry *VldbEntry;
@@ -3196,7 +3200,9 @@ ChangeIPAddr(ipaddr1, ipaddr2, atrans)
 	for (blockindex = NextEntry(atrans, 0, &tentry, &count); blockindex;
 	     blockindex = NextEntry(atrans, blockindex, &tentry, &count)) {
 	    if (++pollcount > 50) {
+#ifndef AFS_PTHREAD_ENV
 		IOMGR_Poll();
+#endif
 		pollcount = 0;
 	    }
 	    for (j = 0; j < NMAXNSERVERS; j++) {
