@@ -18,7 +18,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/voltrans.c,v 1.11.8.1 2007/02/09 01:00:23 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/voltrans.c,v 1.11.8.3 2007/10/30 15:16:59 shadow Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <afs/afsutil.h>
@@ -29,6 +29,7 @@ RCSID
 #include <stdio.h>
 #include <sys/types.h>
 #include <errno.h>
+#include <string.h>
 #ifdef AFS_NT40_ENV
 #include <fcntl.h>
 #include <winsock2.h>
@@ -61,14 +62,6 @@ RCSID
 #include <afs/vnode.h>
 #include <afs/volume.h>
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-
 #include "volser.h"
 
 /*@printflike@*/ extern void Log(const char *format, ...);
@@ -137,21 +130,24 @@ DeleteTrans(register struct volser_trans *atrans, afs_int32 lock)
     register struct volser_trans *tt, **lt;
     afs_int32 error;
 
+    if (lock) VTRANS_LOCK;
     if (atrans->refCount > 1) {
 	/* someone else is using it now */
 	atrans->refCount--;
 	atrans->tflags |= TTDeleted;
+	if (lock) VTRANS_UNLOCK;
 	return 0;
     }
 
     /* otherwise we zap it ourselves */
-    if (lock) VTRANS_LOCK;
     lt = &allTrans;
     for (tt = *lt; tt; lt = &tt->next, tt = *lt) {
 	if (tt == atrans) {
 	    if (tt->volume)
 		VDetachVolume(&error, tt->volume);
 	    tt->volume = NULL;
+	    if (tt->rxCallPtr)
+		rxi_CallError(tt->rxCallPtr, RX_CALL_DEAD);
 	    *lt = tt->next;
 	    free(tt);
 	    if (lock) VTRANS_UNLOCK;
@@ -168,18 +164,22 @@ DeleteTrans(register struct volser_trans *atrans, afs_int32 lock)
 afs_int32 
 TRELE(register struct volser_trans *at)
 {
+    VTRANS_LOCK;
     if (at->refCount == 0) {
 	Log("TRELE: bad refcount\n");
+	VTRANS_UNLOCK;
 	return VOLSERTRELE_ERROR;
     }
 
     at->time = FT_ApproxTime();	/* we're still using it */
     if (at->refCount == 1 && (at->tflags & TTDeleted)) {
-	DeleteTrans(at, 1);
+	DeleteTrans(at, 0);
+	VTRANS_UNLOCK;
 	return 0;
     }
     /* otherwise simply drop refcount */
     at->refCount--;
+    VTRANS_UNLOCK;
     return 0;
 }
 
